@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use JSON::PP;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 sub new {
     my ($class, %opts) = @_;
@@ -78,57 +78,56 @@ sub _traverse {
 
     for my $step (@steps) {
         my $optional = ($step =~ s/\?$//);
-        my @new_stack;
+        my @next_stack;
 
-        if ($step =~ /^(.*?)\[(\d+)\]$/) {
-            my ($key, $index) = ($1, $2);
-            for my $item (@stack) {
+        for my $item (@stack) {
+            next if !defined $item;
+
+            # 添字アクセス: key[index]
+            if ($step =~ /^(.*?)\[(\d+)\]$/) {
+                my ($key, $index) = ($1, $2);
                 if (ref $item eq 'HASH' && exists $item->{$key}) {
                     my $val = $item->{$key};
-                    if (ref $val eq 'ARRAY' && defined $val->[$index]) {
-                        push @new_stack, $val->[$index];
-                    }
-                } elsif ($optional) {
-                    next;
+                    push @next_stack, $val->[$index]
+                        if ref $val eq 'ARRAY' && defined $val->[$index];
                 }
             }
-        }
-        elsif ($step =~ /^(.*?)\[\]$/) {
-            my $key = $1;
-            for my $item (@stack) {
+            # 配列展開: key[]
+            elsif ($step =~ /^(.*?)\[\]$/) {
+                my $key = $1;
                 if (ref $item eq 'HASH' && exists $item->{$key}) {
                     my $val = $item->{$key};
-                    push @new_stack, @$val if ref $val eq 'ARRAY';
-                } elsif ($optional) {
-                    next;
-                }
-            }
-        }
-        else {
-            for my $item (@stack) {
-                if (ref $item eq 'HASH') {
-                    if (exists $item->{$step}) {
-                        push @new_stack, $item->{$step};
-                    } elsif ($optional) {
-                        next;
+                    if (ref $val eq 'ARRAY') {
+                        push @next_stack, @$val;
                     }
-                } elsif (ref $item eq 'ARRAY') {
+                }
+                elsif (ref $item eq 'ARRAY') {
                     for my $sub (@$item) {
-                        if (ref $sub eq 'HASH') {
-                            if (exists $sub->{$step}) {
-                                push @new_stack, $sub->{$step};
-                            } elsif ($optional) {
-                                next;
-                            }
+                        if (ref $sub eq 'HASH' && exists $sub->{$key}) {
+                            my $val = $sub->{$key};
+                            push @next_stack, @$val if ref $val eq 'ARRAY';
+                        }
+                    }
+                }
+            }
+            # 通常アクセス: key
+            else {
+                if (ref $item eq 'HASH' && exists $item->{$step}) {
+                    push @next_stack, $item->{$step};
+                }
+                elsif (ref $item eq 'ARRAY') {
+                    for my $sub (@$item) {
+                        if (ref $sub eq 'HASH' && exists $sub->{$step}) {
+                            push @next_stack, $sub->{$step};
                         }
                     }
-                } elsif ($optional) {
-                    next;
                 }
             }
         }
 
-        @stack = @new_stack;
+        # optional の場合は空でも許容
+        @stack = @next_stack;
+        last if !@stack && !$optional;
     }
 
     return @stack;
@@ -164,7 +163,7 @@ sub _evaluate_condition {
             $value = JSON::PP::true;
         } elsif ($value_raw eq 'false') {
             $value = JSON::PP::false;
-        } elsif ($value_raw =~ /^-?\d+(\.\d+)?$/) {
+        } elsif ($value_raw =~ /^-?\d+(?:\.\d+)?$/) {
             $value = 0 + $value_raw;
         } else {
             $value = $value_raw;
@@ -173,15 +172,18 @@ sub _evaluate_condition {
         my @values = _traverse($item, $path);
         my $field_val = $values[0];
 
-        return eval {
-            return 0 unless defined $field_val;
-            my $is_number = ($field_val =~ /^-?\d+(\.\d+)?$/) && ($value =~ /^-?\d+(\.\d+)?$/);
+        return 0 unless defined $field_val;
 
-            if ($op eq '==') {
-                return $is_number ? $field_val == $value : $field_val eq $value;
-            } elsif ($op eq '!=') {
-                return $is_number ? $field_val != $value : $field_val ne $value;
-            } elsif ($op eq '>') {
+        my $is_number = (!ref($field_val) && $field_val =~ /^-?\d+(?:\.\d+)?$/)
+                     && (!ref($value)     && $value     =~ /^-?\d+(?:\.\d+)?$/);
+
+        if ($op eq '==') {
+            return $is_number ? ($field_val == $value) : ($field_val eq $value);
+        } elsif ($op eq '!=') {
+            return $is_number ? ($field_val != $value) : ($field_val ne $value);
+        } elsif ($is_number) {
+            # 数値比較は数値でしか行わない
+            if ($op eq '>') {
                 return $field_val > $value;
             } elsif ($op eq '>=') {
                 return $field_val >= $value;
@@ -190,8 +192,7 @@ sub _evaluate_condition {
             } elsif ($op eq '<=') {
                 return $field_val <= $value;
             }
-            return 0;
-        } || 0;
+        }
     }
 
     return 0;
@@ -208,7 +209,7 @@ JQ::Lite - A lightweight jq-like JSON query engine in Perl
 
 =head1 VERSION
 
-Version 0.05
+Version 0.07
 
 =head1 SYNOPSIS
 
